#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
author: Francisco Munoz (@fmunoz)
description:
  - Get all networks _appliance _traffic _shaping _custom _performance _classes.
  - Get networks _appliance _traffic _shaping _custom _performance _classes by id.
  - List all custom performance classes for an MX network.
  - Return a custom performance class for an MX network.
extends_documentation_fragment:
  - cisco.meraki.module_info
module: networks_appliance_traffic_shaping_custom_performance_classes_info
notes:
  - SDK Method used are appliance.Appliance.get_network_appliance_traffic_shaping_custom_performance_class,
    appliance.Appliance.get_network_appliance_traffic_shaping_custom_performance_classes,
  - Paths used are get /networks/{networkId}/appliance/trafficShaping/customPerformanceClasses,
    get /networks/{networkId}/appliance/trafficShaping/customPerformanceClasses/{customPerformanceClassId},
options:
  customPerformanceClassId:
    description:
      - CustomPerformanceClassId path parameter. Custom performance class ID.
    type: str
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - description: Complete reference of the getNetworkApplianceTrafficShapingCustomPerformanceClass
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-appliance-traffic-shaping-custom-performance-class
    name: Cisco Meraki documentation for appliance getNetworkApplianceTrafficShapingCustomPerformanceClass
  - description: Complete reference of the getNetworkApplianceTrafficShapingCustomPerformanceClasses
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-appliance-traffic-shaping-custom-performance-classes
    name: Cisco Meraki documentation for appliance getNetworkApplianceTrafficShapingCustomPerformanceClasses
short_description: Information module for networks _appliance _traffic _shaping _custom
  _performance _classes
version_added: 2.16.0
"""

EXAMPLES = r"""
- name: Get all networks _appliance _traffic _shaping _custom _performance _classes
  cisco.meraki.networks_appliance_traffic_shaping_custom_performance_classes_info:
    meraki_api_key: '{{ meraki_api_key }}'
    meraki_base_url: '{{ meraki_base_url }}'
    meraki_single_request_timeout: '{{ meraki_single_request_timeout }}'
    meraki_certificate_path: '{{ meraki_certificate_path }}'
    meraki_requests_proxy: '{{ meraki_requests_proxy }}'
    meraki_wait_on_rate_limit: '{{ meraki_wait_on_rate_limit }}'
    meraki_nginx_429_retry_wait_time: '{{ meraki_nginx_429_retry_wait_time }}'
    meraki_action_batch_retry_wait_time: '{{ meraki_action_batch_retry_wait_time }}'
    meraki_retry_4xx_error: '{{ meraki_retry_4xx_error }}'
    meraki_retry_4xx_error_wait_time: '{{ meraki_retry_4xx_error_wait_time }}'
    meraki_maximum_retries: '{{ meraki_maximum_retries }}'
    meraki_output_log: '{{ meraki_output_log }}'
    meraki_log_file_prefix: '{{ meraki_log_file_prefix }}'
    meraki_log_path: '{{ meraki_log_path }}'
    meraki_print_console: '{{ meraki_print_console }}'
    meraki_suppress_logging: '{{ meraki_suppress_logging }}'
    meraki_simulate: '{{ meraki_simulate }}'
    meraki_be_geo_id: '{{ meraki_be_geo_id }}'
    meraki_use_iterator_for_get_pages: '{{ meraki_use_iterator_for_get_pages }}'
    meraki_inherit_logging_config: '{{ meraki_inherit_logging_config }}'
    networkId: string
  register: result
- name: Get networks _appliance _traffic _shaping _custom _performance _classes by
    id
  cisco.meraki.networks_appliance_traffic_shaping_custom_performance_classes_info:
    meraki_api_key: '{{ meraki_api_key }}'
    meraki_base_url: '{{ meraki_base_url }}'
    meraki_single_request_timeout: '{{ meraki_single_request_timeout }}'
    meraki_certificate_path: '{{ meraki_certificate_path }}'
    meraki_requests_proxy: '{{ meraki_requests_proxy }}'
    meraki_wait_on_rate_limit: '{{ meraki_wait_on_rate_limit }}'
    meraki_nginx_429_retry_wait_time: '{{ meraki_nginx_429_retry_wait_time }}'
    meraki_action_batch_retry_wait_time: '{{ meraki_action_batch_retry_wait_time }}'
    meraki_retry_4xx_error: '{{ meraki_retry_4xx_error }}'
    meraki_retry_4xx_error_wait_time: '{{ meraki_retry_4xx_error_wait_time }}'
    meraki_maximum_retries: '{{ meraki_maximum_retries }}'
    meraki_output_log: '{{ meraki_output_log }}'
    meraki_log_file_prefix: '{{ meraki_log_file_prefix }}'
    meraki_log_path: '{{ meraki_log_path }}'
    meraki_print_console: '{{ meraki_print_console }}'
    meraki_suppress_logging: '{{ meraki_suppress_logging }}'
    meraki_simulate: '{{ meraki_simulate }}'
    meraki_be_geo_id: '{{ meraki_be_geo_id }}'
    meraki_use_iterator_for_get_pages: '{{ meraki_use_iterator_for_get_pages }}'
    meraki_inherit_logging_config: '{{ meraki_inherit_logging_config }}'
    networkId: string
    customPerformanceClassId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "customPerformanceClassId": "string",
      "maxJitter": 0,
      "maxLatency": 0,
      "maxLossPercentage": 0,
      "name": "string"
    }
"""
